@@ -115,6 +115,7 @@ const ChatView = new Lang.Class({
         this._linkCursor = Gdk.Cursor.new(Gdk.CursorType.HAND1);
 
         this._channelSignals = [];
+        this._channel = null;
 
         let roomSignals = [
             { name: 'notify::channel',
@@ -136,6 +137,7 @@ const ChatView = new Lang.Class({
         roomSignals.forEach(Lang.bind(this, function(signal) {
             this._roomSignals.push(room.connect(signal.name, signal.handler));
         }));
+        this._onChannelChanged();
     },
 
     _createTags: function() {
@@ -246,7 +248,7 @@ const ChatView = new Lang.Class({
 
     _onDestroy: function() {
         for (let i = 0; i < this._channelSignals.length; i++)
-            this._room.channel.disconnect(this._channelSignals[i]);
+            this._channel.disconnect(this._channelSignals[i]);
         this._channelSignals = [];
 
         for (let i = 0; i < this._roomSignals.length; i++)
@@ -354,16 +356,16 @@ const ChatView = new Lang.Class({
             this.widget.vadjustment.value != 0 ||
             dir != Gdk.ScrollDirection.UP ||
             this._logWalker.is_end())
-            return false;
+            return Gdk.EVENT_PROPAGATE;
 
         this._fetchingBacklog = true;
         Mainloop.timeout_add(500, Lang.bind(this,
             function() {
                 this._logWalker.get_events_async(NUM_LOG_EVENTS,
                                                  Lang.bind(this, this._onLogEventsReady));
-                return false;
+                return GLib.SOURCE_REMOVE;
             }));
-        return false;
+        return Gdk.EVENT_PROPAGATE;
     },
 
     _onValueChanged: function() {
@@ -374,7 +376,7 @@ const ChatView = new Lang.Class({
             function() {
                 this._checkMessages();
                 this._valueChangedId = 0;
-                return false;
+                return GLib.SOURCE_REMOVE;
             }));
     },
 
@@ -389,7 +391,7 @@ const ChatView = new Lang.Class({
     _handleLinkClicks: function(view, event) {
         let [, button] = event.get_button();
         if (button != Gdk.BUTTON_PRIMARY)
-            return false;
+            return Gdk.EVENT_PROPAGATE;
 
         let isPress = event.get_event_type() == Gdk.EventType.BUTTON_PRESS;
 
@@ -407,17 +409,17 @@ const ChatView = new Lang.Class({
             if (url) {
                 if (isPress) {
                     this._clickedUrl = url;
-                    return true;
+                    return Gdk.EVENT_STOP;
                 } else if (this._clickedUrl == url) {
                     if (url.indexOf(':') == -1)
                         url = 'http://' + url;
                     Gio.AppInfo.launch_default_for_uri(url, null);
-                    return true;
+                    return Gdk.EVENT_STOP;
                 }
                 break;
             }
         }
-        return false;
+        return Gdk.EVENT_PROPAGATE;
     },
 
     _handleLinkHovers: function(view, event) {
@@ -436,16 +438,16 @@ const ChatView = new Lang.Class({
             let cursor = this._hoveringLink ? this._linkCursor : null;
             this._view.get_window(Gtk.TextWindowType.TEXT).set_cursor(cursor);
         }
-        return false;
+        return Gdk.EVENT_PROPAGATE;
     },
 
     _checkMessages: function() {
-        if (!this._active || !this._toplevelFocus || !this._room.channel)
+        if (!this._active || !this._toplevelFocus || !this._channel)
             return;
 
         this._needsIndicator = true;
 
-        let pending = this._room.channel.dup_pending_messages();
+        let pending = this._channel.dup_pending_messages();
         if (pending.length == 0)
             return;
 
@@ -455,25 +457,31 @@ const ChatView = new Lang.Class({
             let [id,] = pending[i].get_pending_message_id();
             let mark = this._pending[id];
             if (!mark) {
-                this._room.channel.ack_message_async(pending[i], null);
+                this._channel.ack_message_async(pending[i], null);
                 continue;
             }
             let iter = buffer.get_iter_at_mark(mark);
             let iterRect = this._view.get_iter_location(iter);
             if (rect.y <= iterRect.y && rect.y + rect.height > iterRect.y)
-                this._room.channel.ack_message_async(pending[i], null);
+                this._channel.ack_message_async(pending[i], null);
         }
     },
 
     _onChannelChanged: function() {
-        if (!this._room.channel) {
-            this._channelSignals = [];
+        if (this._channel == this._room.channel)
             return;
+
+        if (this._channel) {
+            for (let i = 0; i < this._channelSignals.length; i++)
+                this._channel.disconnect(this._channelSignals[i]);
+            this._channelSignals = [];
         }
 
-        for (let i = 0; i < this._channelSignals.length; i++)
-            this._room.channel.disconnect(this._channelSignals[i]);
-        this._channelSignals = [];
+        this._channel = this._room.channel;
+
+        if (!this._channel)
+            return;
+
 
         let channelSignals = [
             { name: 'message-received',
@@ -484,14 +492,11 @@ const ChatView = new Lang.Class({
               handler: Lang.bind(this, this._pendingMessageRemoved) }
         ];
         channelSignals.forEach(Lang.bind(this, function(signal) {
-            this._channelSignals.push(this._room.channel.connect(signal.name, signal.handler));
+            this._channelSignals.push(this._channel.connect(signal.name, signal.handler));
         }));
 
-        this._room.channel.dup_pending_messages().forEach(Lang.bind(this,
+        this._channel.dup_pending_messages().forEach(Lang.bind(this,
             function(message) {
-                let [id, ] = message.get_pending_message_id();
-                if (this._pending[id])
-                    return;
                 this._insertTpMessage(this._room, message);
             }));
         this._checkMessages();
@@ -555,66 +560,62 @@ const ChatView = new Lang.Class({
         let format;
         let desktopSettings = new Gio.Settings({ schema: 'org.gnome.desktop.interface' });
         let clockFormat = desktopSettings.get_string('clock-format');
+        let hasAmPm = date.format('%p') != '';
 
-        switch (clockFormat) {
-            case '24h':
-                if(daysAgo < 1) { // today
-                    /* Translators: Time in 24h format */
-                    format = _("%H\u2236%M");
-                } else if(daysAgo <2) { // yesterday
-                    /* Translators: this is the word "Yesterday" followed by a
-                     time string in 24h format. i.e. "Yesterday, 14:30" */
-                    // xgettext:no-c-format
-                    format = _("Yesterday, %H\u2236%M");
-                } else if (daysAgo < 7) { // this week
-                    /* Translators: this is the week day name followed by a time
-                     string in 24h format. i.e. "Monday, 14:30" */
-                    // xgettext:no-c-format
-                    format = _("%A, %H\u2236%M");
-                } else if (date.get_year() == now.get_year()) { // this year
-                    /* Translators: this is the month name and day number
-                     followed by a time string in 24h format.
-                     i.e. "May 25, 14:30" */
-                    // xgettext:no-c-format
-                    format = _("%B %d, %H\u2236%M");
-                } else { // before this year
-                    /* Translators: this is the month name, day number, year
-                     number followed by a time string in 24h format.
-                     i.e. "May 25 2012, 14:30" */
-                    // xgettext:no-c-format
-                    format = _("%B %d %Y, %H\u2236%M");
-                }
-                break;
-        default:
-            // explicit fall-through
-            case '12h':
-                if(daysAgo < 1) { // today
-                    /* Translators: Time in 12h format */
-                    format = _("%l\u2236%M %p");
-                } else if(daysAgo <2) { // yesterday
-                    /* Translators: this is the word "Yesterday" followed by a
-                     time string in 12h format. i.e. "Yesterday, 2:30 pm" */
-                    // xgettext:no-c-format
-                    format = _("Yesterday, %l\u2236%M %p");
-                } else if (daysAgo < 7) { // this week
-                    /* Translators: this is the week day name followed by a time
-                     string in 12h format. i.e. "Monday, 2:30 pm" */
-                    // xgettext:no-c-format
-                    format = _("%A, %l\u2236%M %p");
-                } else if (date.get_year() == now.get_year()) { // this year
-                    /* Translators: this is the month name and day number
-                     followed by a time string in 12h format.
-                     i.e. "May 25, 2:30 pm" */
-                    // xgettext:no-c-format
-                    format = _("%B %d, %l\u2236%M %p");
-                } else { // before this year
-                    /* Translators: this is the month name, day number, year
-                     number followed by a time string in 12h format.
-                     i.e. "May 25 2012, 2:30 pm"*/
-                    // xgettext:no-c-format
-                    format = _("%B %d %Y, %l\u2236%M %p");
-                }
-                break;
+        if (clockFormat == '24h' || !hasAmPm) {
+            if(daysAgo < 1) { // today
+                /* Translators: Time in 24h format */
+                format = _("%H\u2236%M");
+            } else if(daysAgo <2) { // yesterday
+                /* Translators: this is the word "Yesterday" followed by a
+                 time string in 24h format. i.e. "Yesterday, 14:30" */
+                // xgettext:no-c-format
+                format = _("Yesterday, %H\u2236%M");
+            } else if (daysAgo < 7) { // this week
+                /* Translators: this is the week day name followed by a time
+                 string in 24h format. i.e. "Monday, 14:30" */
+                // xgettext:no-c-format
+                format = _("%A, %H\u2236%M");
+            } else if (date.get_year() == now.get_year()) { // this year
+                /* Translators: this is the month name and day number
+                 followed by a time string in 24h format.
+                 i.e. "May 25, 14:30" */
+                // xgettext:no-c-format
+                format = _("%B %d, %H\u2236%M");
+            } else { // before this year
+                /* Translators: this is the month name, day number, year
+                 number followed by a time string in 24h format.
+                 i.e. "May 25 2012, 14:30" */
+                // xgettext:no-c-format
+                format = _("%B %d %Y, %H\u2236%M");
+            }
+        } else {
+            if(daysAgo < 1) { // today
+                /* Translators: Time in 12h format */
+                format = _("%l\u2236%M %p");
+            } else if(daysAgo <2) { // yesterday
+                /* Translators: this is the word "Yesterday" followed by a
+                 time string in 12h format. i.e. "Yesterday, 2:30 pm" */
+                // xgettext:no-c-format
+                format = _("Yesterday, %l\u2236%M %p");
+            } else if (daysAgo < 7) { // this week
+                /* Translators: this is the week day name followed by a time
+                 string in 12h format. i.e. "Monday, 2:30 pm" */
+                // xgettext:no-c-format
+                format = _("%A, %l\u2236%M %p");
+            } else if (date.get_year() == now.get_year()) { // this year
+                /* Translators: this is the month name and day number
+                 followed by a time string in 12h format.
+                 i.e. "May 25, 2:30 pm" */
+                // xgettext:no-c-format
+                format = _("%B %d, %l\u2236%M %p");
+            } else { // before this year
+                /* Translators: this is the month name, day number, year
+                 number followed by a time string in 12h format.
+                 i.e. "May 25 2012, 2:30 pm"*/
+                // xgettext:no-c-format
+                format = _("%B %d %Y, %l\u2236%M %p");
+            }
         }
 
         return date.format(format);
@@ -659,7 +660,7 @@ const ChatView = new Lang.Class({
         let buffer = this._view.get_buffer();
         if (!valid /* outgoing */ ||
             (this._active && this._toplevelFocus && this._nPending == 0)) {
-            this._room.channel.ack_message_async(tpMessage, null);
+            this._channel.ack_message_async(tpMessage, null);
         } else if (message.shouldHighlight || this._needsIndicator) {
             let iter = buffer.get_end_iter();
 
