@@ -42,6 +42,15 @@ const _ChatroomManager = new Lang.Class({
             this._app.release(); // no point in carrying on
         }
 
+        let joinAction = this._app.lookup_action('join-room');
+        joinAction.connect('activate', Lang.bind(this, this._onJoinActivated));
+
+        let queryAction = this._app.lookup_action('message-user');
+        queryAction.connect('activate', Lang.bind(this, this._onQueryActivated));
+
+        let leaveAction = this._app.lookup_action('leave-room');
+        leaveAction.connect('activate', Lang.bind(this, this._onLeaveActivated));
+
         this._observer = Tp.SimpleObserver.new_with_am(am, true,
             'Polari', true, Lang.bind(this, this._observeChannels));
 
@@ -70,6 +79,10 @@ const _ChatroomManager = new Lang.Class({
 
         am.connect('account-enabled',
                    Lang.bind(this, this._restoreSavedChannels));
+        am.connect('account-disabled',
+                   Lang.bind(this, this._onAccountDisabled));
+        am.connect('account-removed',
+                   Lang.bind(this, this._onAccountDisabled));
         this._restoreSavedChannels();
 
         this._networkMonitor.connect('notify::network-available', Lang.bind(this,
@@ -79,6 +92,14 @@ const _ChatroomManager = new Lang.Class({
             }));
     },
 
+    _onAccountDisabled: function(am, account) {
+        for (let id in this._rooms) {
+            let room = this._rooms[id];
+            if (room.account == account)
+                this._removeRoom(room);
+        }
+    },
+
     _restoreSavedChannels: function() {
         let settings = new Gio.Settings({ schema: 'org.gnome.polari' });
         let savedChannels = settings.get_value('saved-channel-list').deep_unpack();
@@ -100,21 +121,55 @@ const _ChatroomManager = new Lang.Class({
         action.activate(parameter);
     },
 
-    _ensureRoomForChannel: function(channel) {
-        let room = this._rooms[channel.get_object_path()];
+    _onJoinActivated: function(action, parameter) {
+        let [accountPath, channelName, ] = parameter.deep_unpack();
+        let factory = Tp.AccountManager.dup().get_factory();
+        let account = factory.ensure_account(accountPath, []);
+
+        if (!account.enabled)
+            return;
+
+        this._ensureRoom(account, channelName, Tp.HandleType.ROOM);
+    },
+
+    _onQueryActivated: function(action, parameter) {
+        let [accountPath, channelName, ] = parameter.deep_unpack();
+        let factory = Tp.AccountManager.dup().get_factory();
+        let account = factory.ensure_account(accountPath, []);
+
+        if (!account.enabled)
+            return;
+
+        this._ensureRoom(account, channelName, Tp.HandleType.CONTACT);
+    },
+
+    _onLeaveActivated: function(action, parameter) {
+        let [id, ] = parameter.deep_unpack();
+        let room = this._rooms[id];
+        this._removeRoom(room);
+    },
+
+    _ensureRoom: function(account, channelName, type) {
+        let room = this._rooms[Polari.create_room_id(account, channelName, type)];
         if (room)
             return room;
 
-        let room = new Polari.Room({ channel: channel });
-        room.channel.connect('invalidated', Lang.bind(this,
-            function() {
-                this._removeRoom(room);
-            }));
+        let room = new Polari.Room({ account: account,
+                                     channel_name: channelName,
+                                     type: type });
         this._addRoom(room);
 
         return room;
     },
 
+    _ensureRoomForChannel: function(channel) {
+        let account = channel.connection.get_account();
+        let channelName = channel.identifier;
+        let room = this._ensureRoom(account, channelName, channel.handle_type);
+        room.channel = channel;
+        return room;
+    },
+
     _processRequest: function(context, connection, channels, processChannel) {
         if (connection.protocol_name != 'irc') {
             let message = 'Not implementing non-IRC protocols';
@@ -172,13 +227,25 @@ const _ChatroomManager = new Lang.Class({
     _addRoom: function(room) {
         if (this._rooms[room.id])
             return;
+
+        room._channelChangedId = room.connect('notify::channel', Lang.bind(this,
+            function(room) {
+                if (room == this._activeRoom)
+                    this.emit('active-state-changed');
+            }));
+
         this._rooms[room.id] = room;
         this.emit('room-added', room);
+
+        if (this.roomCount == 1)
+            this.setActiveRoom(room);
     },
 
     _removeRoom: function(room) {
         if (!this._rooms[room.id])
             return;
+        room.disconnect(room._channelChangedId);
+        delete room._channelChangedId;
         delete this._rooms[room.id];
         this.emit('room-removed', room);
     },
@@ -189,6 +256,7 @@ const _ChatroomManager = new Lang.Class({
 
         this._activeRoom = room;
         this.emit('active-changed', room);
+        this.emit('active-state-changed');
     },
 
     getActiveRoom: function(room) {
